"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var EventEmitter = require("eventemitter3");
var uuid = require("uuid");

var Node = function () {
  function Node(options, eventemitter) {
    _classCallCheck(this, Node);

    this.id = uuid.v1();
    this.options = options;
    this.ee = eventemitter ? eventemitter : new EventEmitter();
    this.observers = [];
    this.upstreams = {};
    this.downstreams = {};
    this.meta = {};
  }

  _createClass(Node, [{
    key: "push",
    value: function push(signal) {
      var _this = this;

      var interruptible = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

      if (interruptible === false || !this.isInterruptibleSignal(signal)) {
        this.onReceive(signal);
        return this;
      }

      setImmediate(function () {
        _this.onReceive(signal);
      });
      return this;
    }
  }, {
    key: "onReceive",
    value: function onReceive(signal) {
      this.invokeObservers("onReceive", signal);

      if (this.isErrorSignal(signal)) {
        this.onError(signal);
        this.request();
      } else if (this.isEndSignal(signal)) {
        try {
          this.onEnd(signal);
        } catch (error) {
          this.throwError(error, signal);
        }
      } else {
        try {
          this.onSignal(signal);
        } catch (error) {
          this.throwError(error, signal);
        }
      }
      return this;
    }
  }, {
    key: "onSignal",
    value: function onSignal(signal) {
      this.send(signal);
      return this;
    }
  }, {
    key: "onError",
    value: function onError(error) {
      this.send(error);
      return this;
    }
  }, {
    key: "onEnd",
    value: function onEnd(signal) {
      this.send(signal);
      return this;
    }
  }, {
    key: "send",
    value: function send(signal) {
      var _this2 = this;

      var interruptible = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

      this.invokeObservers("send", signal);
      if (interruptible === false || !this.isInterruptibleSignal(signal)) {
        for (var id in this.downstreams) {
          this.downstreams[id].onReceive(signal);
        }
        return this;
      }
      setImmediate(function () {
        _this2.ee.emit("outgoing-" + _this2.id, signal);
      });
      return this;
    }
  }, {
    key: "observe",
    value: function observe(observer) {
      this.observers.push(observer);
      return this;
    }
  }, {
    key: "to",
    value: function to(downstream) {
      var _this3 = this;

      this.invokeObservers("to", downstream);

      this.downstreams[downstream.id] = downstream;
      this.ee.on("outgoing-" + this.id, function (signal) {
        downstream.push(signal);
      });
      // for pull model
      downstream.ee.on("request-" + downstream.id, function (cmd) {
        _this3.pull(cmd);
      });
      downstream.from(this);
      return downstream;
    }
  }, {
    key: "pull",
    value: function pull(cmd) {
      var _this4 = this;

      setImmediate(function () {
        _this4.onRequest(cmd);
      });
      return this;
    }
  }, {
    key: "onRequest",
    value: function onRequest(cmd) {
      this.invokeObservers("onRequest", cmd);
      this.request(cmd);
      return this;
    }
  }, {
    key: "request",
    value: function request(cmd) {
      var _this5 = this;

      this.invokeObservers("request", cmd);
      setImmediate(function () {
        _this5.ee.emit("request-" + _this5.id, cmd);
      });
      return this;
    }
  }, {
    key: "from",
    value: function from(upstream) {
      this.upstreams[upstream.id] = upstream;
      this.invokeObservers("from", upstream);
      return this;
    }
  }, {
    key: "isInterruptibleSignal",
    value: function isInterruptibleSignal(signal) {
      return true;
    }
  }, {
    key: "isErrorSignal",
    value: function isErrorSignal(signal) {
      return signal instanceof Error;
    }
  }, {
    key: "isEndSignal",
    value: function isEndSignal(signal) {
      //$FlowIgnore
      return signal === Node.END;
    }
  }, {
    key: "throwError",
    value: function throwError(error, signal) {
      //$FlowIgnore
      error.signal = signal;
      return this.send(error);
    }
  }, {
    key: "invokeObservers",
    value: function invokeObservers(when) {
      var _this6 = this;

      for (var _len = arguments.length, data = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        data[_key - 1] = arguments[_key];
      }

      data.unshift(when);
      data.unshift(this);
      try {
        // run globally registered observers, first
        var globalObservers = require("./index").observers;
        for (var name in globalObservers) {
          globalObservers[name].apply(this, data);
        };

        this.observers.forEach(function (fn) {
          fn.apply(_this6, data);
        });
      } catch (err) {
        console.error(err.message);
      }
    }
  }]);

  return Node;
}();

//$FlowIgnore


Node.END = "__SIGNAL_END__";

module.exports = Node;