"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Node = require("./Node");

function map(fn) {
  var MapNode = function (_Node) {
    _inherits(MapNode, _Node);

    function MapNode(options, eventemitter) {
      _classCallCheck(this, MapNode);

      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(MapNode).call(this, options, eventemitter));

      _this.fn = options;
      return _this;
    }

    _createClass(MapNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        var out = this.fn(signal);
        this.send(out);
      }
    }]);

    return MapNode;
  }(Node);

  return new MapNode(fn);
}

exports["map"] = map;

function filter(fn) {
  var FilterNode = function (_Node2) {
    _inherits(FilterNode, _Node2);

    function FilterNode(options, eventemitter) {
      _classCallCheck(this, FilterNode);

      var _this2 = _possibleConstructorReturn(this, Object.getPrototypeOf(FilterNode).call(this, options, eventemitter));

      _this2.fn = options;
      return _this2;
    }

    _createClass(FilterNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        if (fn(signal)) {
          this.send(signal);
        } else {
          // request next signal
          this.request();
        }
      }
    }]);

    return FilterNode;
  }(Node);

  return new FilterNode(fn);
}

exports["filter"] = filter;

function reduce(memo, iterator) {
  var ReduceNode = function (_Node3) {
    _inherits(ReduceNode, _Node3);

    function ReduceNode(options, eventemitter) {
      _classCallCheck(this, ReduceNode);

      var _this3 = _possibleConstructorReturn(this, Object.getPrototypeOf(ReduceNode).call(this, options, eventemitter));

      _this3.memo = _this3.options.memo;
      _this3.iterator = _this3.options.iterator;
      _this3.END = false;
      return _this3;
    }

    _createClass(ReduceNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        this.memo = this.iterator.call(this, this.memo, signal);
        this.request(); // back pressure call for next
      }
    }, {
      key: "onEnd",
      value: function onEnd() {
        this.send(this.memo);
        this.END = true;
      }
    }, {
      key: "onRequest",
      value: function onRequest(cmd) {
        if (!this.END) {
          this.request();
        } else {
          this.send(Node.END);
        }
      }
    }]);

    return ReduceNode;
  }(Node);

  return new ReduceNode({ memo: memo, iterator: iterator });
}

exports["reduce"] = reduce;

function errors(fn) {
  var ErrorsNode = function (_Node4) {
    _inherits(ErrorsNode, _Node4);

    function ErrorsNode(options, eventemitter) {
      _classCallCheck(this, ErrorsNode);

      var _this4 = _possibleConstructorReturn(this, Object.getPrototypeOf(ErrorsNode).call(this, options, eventemitter));

      _this4.fn = options;
      return _this4;
    }

    _createClass(ErrorsNode, [{
      key: "onError",
      value: function onError(error) {
        var _this5 = this;

        this.fn(error, function (signal) {
          _this5.send(signal);
        });
      }
    }]);

    return ErrorsNode;
  }(Node);

  return new ErrorsNode(fn);
};

exports["errors"] = errors;

function sink() {
  var SinkNode = function (_Node5) {
    _inherits(SinkNode, _Node5);

    function SinkNode() {
      _classCallCheck(this, SinkNode);

      return _possibleConstructorReturn(this, Object.getPrototypeOf(SinkNode).apply(this, arguments));
    }

    _createClass(SinkNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        this.request();
      }
    }, {
      key: "from",
      value: function from(node) {
        this.request();
      }
    }]);

    return SinkNode;
  }(Node);

  return new SinkNode();
};

exports["sink"] = sink;

function throttle(ms) {
  var last = new Date().getTime();

  var ThrottleNode = function (_Node6) {
    _inherits(ThrottleNode, _Node6);

    function ThrottleNode() {
      _classCallCheck(this, ThrottleNode);

      return _possibleConstructorReturn(this, Object.getPrototypeOf(ThrottleNode).apply(this, arguments));
    }

    _createClass(ThrottleNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        var now = new Date().getTime();
        if (now - ms >= last) {
          last = now;
          this.send(signal);
        }
      }
    }]);

    return ThrottleNode;
  }(Node);

  return new ThrottleNode(ms);
};

exports["throttle"] = throttle;

function scan(n, add) {
  var ScanNode = function (_Node7) {
    _inherits(ScanNode, _Node7);

    function ScanNode(options, eventemitter) {
      _classCallCheck(this, ScanNode);

      var _this8 = _possibleConstructorReturn(this, Object.getPrototypeOf(ScanNode).call(this, options, eventemitter));

      _this8.n = options.n;
      _this8.add = options.add;
      return _this8;
    }

    _createClass(ScanNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        this.n = this.add.call(this, this.n, signal);
        this.send(this.n);
      }
    }]);

    return ScanNode;
  }(Node);

  return new ScanNode({
    n: n,
    add: add
  });
};

exports["scan"] = scan;

function act(fn) {
  var ActNode = function (_Node8) {
    _inherits(ActNode, _Node8);

    function ActNode(options, eventemitter) {
      _classCallCheck(this, ActNode);

      var _this9 = _possibleConstructorReturn(this, Object.getPrototypeOf(ActNode).call(this, options, eventemitter));

      _this9.fn = options;
      return _this9;
    }

    _createClass(ActNode, [{
      key: "onSignal",
      value: function onSignal(signal) {
        this.fn(signal);
        this.send(signal);
      }
    }]);

    return ActNode;
  }(Node);

  return new ActNode(fn);
}

exports["act"] = act;

function done(fn) {
  var DoneNode = function (_Node9) {
    _inherits(DoneNode, _Node9);

    function DoneNode(options, eventemitter) {
      _classCallCheck(this, DoneNode);

      var _this10 = _possibleConstructorReturn(this, Object.getPrototypeOf(DoneNode).call(this, options, eventemitter));

      _this10.fn = options;
      return _this10;
    }

    _createClass(DoneNode, [{
      key: "onEnd",
      value: function onEnd(signal) {
        this.fn(signal);
      }
    }]);

    return DoneNode;
  }(Node);

  return new DoneNode(fn);
}

exports["done"] = done;